/*
 * Created on 3-nov-2003
 */
package slides;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

/**
 * Classe che consente la lettura e la visualizzazione del contenuto di un file XML; necessita della API Xerces (un famoso parser XML Open Source reperibile su http://xml.apache.org)
 * @author Ignazio Palmisano
*/
public class XMLSimpleProgram {
	/** il documento XML incapsulato da questa classe*/
	private Document doc;
	/** un riferimento a una DocumentBuilderFactory, vale a dire una fabbrica di costruttori per documenti (XML, ovvio...);  statica perch non  necessario averne una diversa per ogni oggetto, ma ne occorre una sola*/
	private static DocumentBuilderFactory factory;
	/** un builder, o costruttore per documenti XML; anche questo  statico perch ce ne occorre solo uno*/
	private static DocumentBuilder builder;
	
	/**questo blocco di codice viene eseguito la prima volta che si referenzia la classe; inizializza solo variabili statiche*/
	static{
		//inizializzazione della factory
		factory=DocumentBuilderFactory.newInstance();
		try {
			// ottenimento di un builder
			builder=factory.newDocumentBuilder();
		}
		catch (ParserConfigurationException e) {
			// TODO Auto-generated catch block
			// errore di configurazione del parser....
			e.printStackTrace();
		}
	}
	/**costruttore di default; quesito:  necessario?*/
	public XMLSimpleProgram(){
	}
	/**permette di settare l'input da cui caricare il documento; un InputStream  un modo generico per indicare un input proveniente da uno stream (file, socket, periferiche audio)*/
	public void setInput(InputStream stream){
		try {
			// parsing del documento XML; in questo punto potranno essere sollevate eccezioni se il file  scritto male
			this.doc=builder.parse(stream);
		}
		catch (SAXException e) {
			// TODO Auto-generated catch block
			// file mal formato
			e.printStackTrace();
		}
		catch (IOException e) {
			// TODO Auto-generated catch block
			// file inesistente o corrotto
			e.printStackTrace();
		}
		catch (Exception e){
			// eccezione generica
			e.printStackTrace();
		}
	}
	/**restituisce il doc contenuto in questa classe e consente di manipolarlo*/
	public Document getDocument(){
		return this.doc;
	}

	public static void main(String[] args) {
		// http://www.w3.org/TR/2004/REC-DOM-Level-3-LS-20040407/
		// in attesa di usare una VM che implementi questo DOM
		// JDK 1.5...

		// questo oggetto permette di scegliere alcune caratteristiche per formattare l'output
		OutputFormat formatPrettyPrint=new OutputFormat();
		formatPrettyPrint.setIndenting(true);
		OutputFormat formatNoPrettyPrint=new OutputFormat();
		formatNoPrettyPrint.setIndenting(false);
		// questo oggetto consente di serializzare il documento XML (cio permette di scriverlo su file o sulla console) utilizzando il pretty printing
		XMLSerializer indentedSerializer= new XMLSerializer(System.out, formatPrettyPrint);
		// questo invece lo fa senza pretty printing... metodo + corretto, in generale
		XMLSerializer betterSerializer= new XMLSerializer(System.out, formatNoPrettyPrint);

		XMLSimpleProgram program=new XMLSimpleProgram();
		try {
			System.out.println("\n\nPrimo file con prettyprint");
			// lettura del file passato come primo argomento
			program.setInput(new FileInputStream(args[0]));
			// ottiene il nodo radice del documento XML e lo stampa sullo standard output 
			indentedSerializer.asDOMSerializer().serialize(program.getDocument().getDocumentElement());
			
			System.out.println("\n\nSecondo file con prettyprint");
			program.setInput(new FileInputStream(args[1]));
			// ottiene il nodo radice del documento XML e lo stampa sullo standard output 
			indentedSerializer.asDOMSerializer().serialize(program.getDocument().getDocumentElement());
			System.out.println("\n\nPrimo file senza prettyprint");
			// lettura del file passato come primo argomento
			program.setInput(new FileInputStream(args[0]));
			// ottiene il nodo radice del documento XML e lo stampa sullo standard output 
			betterSerializer.asDOMSerializer().serialize(program.getDocument().getDocumentElement());
			
			System.out.println("\n\nSecondo file senza prettyprint");
			program.setInput(new FileInputStream(args[1]));
			System.out.println();
			// ottiene il nodo radice del documento XML e lo stampa sullo standard output 
			betterSerializer.asDOMSerializer().serialize(program.getDocument().getDocumentElement());
			System.out.println("\n\n");
			}
		catch (FileNotFoundException e) {
			System.out.println("ATTENZIONE: il file specificato non  stato trovato!");
			e.printStackTrace();
		}
		catch (IOException e) {
			// questo errore non dovrebbe mai venire fuori...
			System.out.println("ATTENZIONE: errore nrella scrittura!");
			e.printStackTrace();
		}
		catch (ArrayIndexOutOfBoundsException e) {
			System.out.println("Utilizzo: XMLSimpleProgram <nome_file_xml> <nome_file_xml>");
		}
	}
}
